from bnop.core.object_model.objects.bnop_elements import (
    BnopElements,
)


class BnopElementFactories:

    @staticmethod
    def create(
        uuid, owning_repository_uuid
    ):
        bnop_element = BnopElements(
            uuid, owning_repository_uuid
        )

        return bnop_element



from bnop.migrations.bnop_to_xml_migration.common_knowledge.vanilla_bnop_to_xml_migration_configurations import (
    VanillaBnopToXmlMigrationConfigurations,
)
from bnop.migrations.bnop_to_xml_migration.xml_bnop_registers import (
    XmlBnopRegisters,
)
from lxml import etree


def orchestrate_xml_write(
    xml_file_path: str,
    configuration=VanillaBnopToXmlMigrationConfigurations(),
):
    root_element = etree.Element(
        "Model"
    )

    XmlBnopRegisters.initialise_migration_uow()

    for (
        bnop_sorter
    ) in (
        configuration.bnop_object_sorters
    ):
        bnop_sorter()

    for (
        section_creator
    ) in (
        configuration.xml_section_creators
    ):
        section_creator(
            root_element, configuration
        )

    tree = etree.ElementTree(
        element=root_element
    )

    tree.write(
        xml_file_path, pretty_print=True
    )



from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)


def translate_to_uuids(
    bnop_objects: set,
):
    object_uuids = set()

    for bnop_object in bnop_objects:
        object_uuids.add(
            bnop_object.uuid
        )

    return object_uuids


def translate_to_objects(
    bnop_object_uuids: set,
):
    objects = set()

    for (
        bnop_object_uuid
    ) in bnop_object_uuids:
        objects.add(
            BnopObjects.registry_keyed_on_uuid[
                bnop_object_uuid
            ]
        )

    return objects



from bnop.core.factories.bnop_object_factories import (
    BnopObjectFactories,
)
from bnop.core.object_model.objects.bnop_types import (
    BnopTypes,
)


class BnopTypeFactories(
    BnopObjectFactories
):
    pass

    @staticmethod
    def create(
        uuid,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        bnop_type = BnopTypes(
            uuid,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        return bnop_type



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)


class BoroObjectCkIdToBnopMappings(
    object
):
    registry_keyed_on_ckid = {}
    registry_keyed_on_bnop = {}

    def __init__(
        self,
        boro_object_ckid: BoroObjectCkIds,
        bnop_object: BnopObjects,
    ):
        self.ckid = boro_object_ckid

        self.bnop_object = bnop_object

        self.registry_keyed_on_ckid.update(
            {
                self.ckid: self.bnop_object
            }
        )

        self.registry_keyed_on_bnop.update(
            {
                self.bnop_object: self.ckid
            }
        )

    @staticmethod
    def get_bnop_type(
        boro_object_ckid: BoroObjectCkIds,
    ):
        bnop_object = BoroObjectCkIdToBnopMappings.registry_keyed_on_ckid[
            boro_object_ckid
        ]

        return bnop_object

    @staticmethod
    def get_ckid(
        bnop_object: BnopObjects,
    ):
        boro_object_ckid = BoroObjectCkIdToBnopMappings.registry_keyed_on_bnop[
            bnop_object
        ]

        return boro_object_ckid



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)


class XmlBnopRegisters:
    __xml_model_branches = dict()

    __migrated_bnop_objects_keyed_by_ck_id = (
        dict()
    )

    @staticmethod
    def initialise_migration_uow():
        XmlBnopRegisters.__migrated_bnop_objects_keyed_by_ck_id = (
            BnopObjects.registry_keyed_on_ckid_type
        )

    @staticmethod
    def update_migration_uow(
        boro_ckid_type: BoroObjectCkIds,
        bnop_objects: list,
    ):
        XmlBnopRegisters.__migrated_bnop_objects_keyed_by_ck_id[
            boro_ckid_type
        ] = bnop_objects

    @staticmethod
    def get_migration_uow_type(
        boro_ckid_type: BoroObjectCkIds,
    ):
        return XmlBnopRegisters.__migrated_bnop_objects_keyed_by_ck_id[
            boro_ckid_type
        ]

    @staticmethod
    def clear_xml_model_branches():
        XmlBnopRegisters.__xml_model_branches.clear()

    @staticmethod
    def try_to_add_to_branches(
        bnop_object: BnopObjects,
        bnop_parent: BnopObjects,
    ) -> bool:
        success = True

        if (
            bnop_parent
            in XmlBnopRegisters.__xml_model_branches
        ):
            if (
                bnop_object
                in XmlBnopRegisters.__xml_model_branches[
                    bnop_parent
                ]
            ):
                return False

            else:
                XmlBnopRegisters.__xml_model_branches[
                    bnop_parent
                ].append(
                    bnop_object
                )

        else:
            success = XmlBnopRegisters.__try_to_add_to_trunks(
                bnop_object=bnop_object,
                bnop_parent=bnop_parent,
            )

        return success

    @staticmethod
    def __try_to_add_to_trunks(
        bnop_object: BnopObjects,
        bnop_parent: BnopObjects,
    ) -> bool:
        added_to_trunk = False

        for (
            branch_root,
            branch_trunk,
        ) in (
            XmlBnopRegisters.__xml_model_branches.items()
        ):
            added_to_trunk = XmlBnopRegisters.__try_to_add_to_trunk(
                bnop_object=bnop_object,
                bnop_parent=bnop_parent,
                branch_trunk=branch_trunk,
                branch_root=branch_root,
                added_to_trunk=added_to_trunk,
            )

        if not added_to_trunk:
            XmlBnopRegisters.__xml_model_branches[
                bnop_parent
            ] = [
                bnop_object
            ]

        return True

    @staticmethod
    def __try_to_add_to_trunk(
        bnop_object: BnopObjects,
        bnop_parent: BnopObjects,
        branch_trunk: list,
        branch_root: BnopObjects,
        added_to_trunk: bool,
    ) -> bool:
        if bnop_parent in branch_trunk:
            if (
                bnop_object
                in XmlBnopRegisters.__xml_model_branches[
                    branch_root
                ]
            ):
                return False

            else:
                XmlBnopRegisters.__xml_model_branches[
                    branch_root
                ].append(
                    bnop_object
                )

                added_to_trunk = True

        return added_to_trunk



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)
from bnop.migrations.bnop_to_xml_migration.common_knowledge.bnop_to_xml_migration_configurations import (
    BnopToXmlMigrationConfigurations,
)
from lxml import etree
from lxml.etree import Element


def add_bnop_object_to_xml_tree(
    bnop_object: BnopObjects,
    parent_element: Element,
    configuration: BnopToXmlMigrationConfigurations,
):
    if (
        configuration.include_presentation_names
    ):
        object_tree_element = etree.SubElement(
            parent_element,
            BoroObjectCkIds.Objects.name,
            uuid=str(bnop_object.uuid),
            uml_name=bnop_object.uml_name,
        )

    else:
        object_tree_element = etree.SubElement(
            parent_element,
            BoroObjectCkIds.Objects.name,
            uuid=str(bnop_object.uuid),
        )

    add_bnop_object_types_to_xml_tree(
        bnop_object=bnop_object,
        object_tree_element=object_tree_element,
        configuration=configuration,
    )

    add_bnop_object_supertypes_to_xml_tree(
        bnop_object=bnop_object,
        object_tree_element=object_tree_element,
        configuration=configuration,
    )


def add_bnop_object_types_to_xml_tree(
    bnop_object: BnopObjects,
    object_tree_element: Element,
    configuration: BnopToXmlMigrationConfigurations,
):
    if len(bnop_object.types) == 0:
        return

    type_instances_tree_element = etree.SubElement(
        object_tree_element,
        BoroObjectCkIds.TypesInstances.name,
    )

    for bnop_type in bnop_object.types:
        if (
            configuration.include_presentation_names
        ):
            etree.SubElement(
                type_instances_tree_element,
                BoroObjectCkIds.Types.name,
                uuid=str(
                    bnop_type.uuid
                ),
                uml_name=bnop_type.uml_name,
            )

        else:
            etree.SubElement(
                type_instances_tree_element,
                BoroObjectCkIds.Types.name,
                uuid=str(
                    bnop_type.uuid
                ),
            )


def add_bnop_object_supertypes_to_xml_tree(
    bnop_object: BnopObjects,
    object_tree_element: Element,
    configuration: BnopToXmlMigrationConfigurations,
):
    if len(bnop_object.supertypes) == 0:
        return

    supertypes_tree_element = etree.SubElement(
        object_tree_element,
        BoroObjectCkIds.SuperSubTypes.name,
    )

    for (
        bnop_type
    ) in bnop_object.supertypes:
        if (
            configuration.include_presentation_names
        ):
            etree.SubElement(
                supertypes_tree_element,
                BoroObjectCkIds.Types.name,
                uuid=str(
                    bnop_type.uuid
                ),
                uml_name=bnop_type.uml_name,
            )

        else:
            etree.SubElement(
                supertypes_tree_element,
                BoroObjectCkIds.Types.name,
                uuid=str(
                    bnop_type.uuid
                ),
            )



from bnop.core.object_model.objects.bnop_types import (
    BnopTypes,
)


class BnopRepresentations(BnopTypes):

    def __init__(
        self,
        uuid,
        exemplar_representation,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        BnopTypes.__init__(
            self,
            uuid,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        self.exemplar_representation = (
            exemplar_representation
        )



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)


class BnopObjectFactories:
    def __init__(self):
        pass

    @staticmethod
    def create(
        uuid,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        bnop_object = BnopObjects(
            uuid,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        bnop_object.add_to_registry_keyed_on_ckid_type(
            BoroObjectCkIds.Objects
        )

        return bnop_object



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)
from bnop.core.object_model.objects.places.bnop_placed_objects_dictionaries import (
    BnopPlacedObjectsDictionaries,
)


class BnopTuples(BnopObjects):

    def __init__(
        self,
        uuid,
        tuple_placed_objects_dictionary: BnopPlacedObjectsDictionaries,
        immutable_minor_composition_couple_type_boro_object_ckid: BoroObjectCkIds,
        owning_repository_uuid,
    ):
        BnopObjects.__init__(
            self,
            uuid,
            owning_repository_uuid,
        )

        self.add_to_registry_keyed_on_ckid_type(
            immutable_minor_composition_couple_type_boro_object_ckid
        )

        self.immutable_minor_composition_couple_type_boro_object_ckid = immutable_minor_composition_couple_type_boro_object_ckid

        self.tuple_placed_objects_dictionary = tuple_placed_objects_dictionary



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)


class BnopObjects(object):
    registry_keyed_on_uuid = {}

    registry_keyed_on_ckid_type = {}

    matched_objects = []

    def __init__(
        self,
        uuid,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        self.uuid = uuid

        self.registry_keyed_on_uuid.update(
            {self.uuid: self}
        )

        self.owning_repository_uuid = (
            owning_repository_uuid
        )

        self.is_named_bys = set()

        self.names = set()

        self.types = set()

        self.instances = set()

        self.supertypes = set()

        self.subtypes = set()

        self.uml_name = (
            presentation_name
        )

        self.naming_spaces = set()

    def add_to_registry_keyed_on_ckid_type(
        self,
        boro_object_ck_id: BoroObjectCkIds,
    ):

        if (
            boro_object_ck_id
            in self.registry_keyed_on_ckid_type
        ):
            ckid_typed_objects = self.registry_keyed_on_ckid_type[
                boro_object_ck_id
            ]
        else:
            ckid_typed_objects = set()

            self.registry_keyed_on_ckid_type.update(
                {
                    boro_object_ck_id: ckid_typed_objects
                }
            )

        ckid_typed_objects.add(self)

    @staticmethod
    def import_bnop_object(bnop_object):
        BnopObjects.registry_keyed_on_uuid.update(
            {
                bnop_object.uuid: bnop_object
            }
        )

        return bnop_object

    @staticmethod
    def add_to_matched_objects(
        bnop_object,
    ):
        BnopObjects.matched_objects.append(
            bnop_object.uuid
        )



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.migrations.bnop_to_xml_migration.adders.bnop_object_xml_adder import (
    add_bnop_object_to_xml_tree,
)
from bnop.migrations.bnop_to_xml_migration.common_knowledge.bnop_to_xml_migration_configurations import (
    BnopToXmlMigrationConfigurations,
)
from bnop.migrations.bnop_to_xml_migration.xml_bnop_registers import (
    XmlBnopRegisters,
)
from lxml import etree
from lxml.etree import Element


def add_named_objects(
    xml_root: Element,
    configuration: BnopToXmlMigrationConfigurations,
):
    bnop_objects = XmlBnopRegisters.get_migration_uow_type(
        boro_ckid_type=BoroObjectCkIds.Objects
    )

    objects_tree_element = (
        etree.SubElement(
            xml_root, "Objects"
        )
    )

    for bnop_object in bnop_objects:
        if (
            len(
                bnop_object.is_named_bys
            )
            > 0
        ):
            add_bnop_object_to_xml_tree(
                bnop_object=bnop_object,
                parent_element=objects_tree_element,
                configuration=configuration,
            )



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.bnop_facades import (
    BnopFacades,
)
from bnop.core.object_model.objects.bnop_names import (
    BnopNames,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)
from bnop.core.object_model.objects.bnop_types import (
    BnopTypes,
)


def add_name_if_required(
    named_object: BnopObjects,
    name: BnopNames,
    naming_space: BnopTypes,
    owning_repository_uuid,
):
    BnopFacades.create_new_bnop_tuple_from_two_placed_objects(
        naming_space,
        name,
        BoroObjectCkIds.TypesInstances,
        owning_repository_uuid,
    )

    BnopFacades.create_new_bnop_tuple_from_two_placed_objects(
        named_object,
        name,
        BoroObjectCkIds.NamedBy,
        owning_repository_uuid,
    )


def add_exemplar_name_if_required(
    named_object: BnopObjects,
    exemplar_name: str,
    naming_space: BnopTypes,
    owning_repository_uuid,
):
    name = BnopFacades.create_new_bnop_name(
        exemplar_name,
        owning_repository_uuid,
    )

    add_name_if_required(
        named_object,
        name,
        naming_space,
        owning_repository_uuid,
    )



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.core.object_model.objects.bnop_tuples import (
    BnopTuples,
)


class BnopTupleFactories:

    @staticmethod
    def create(
        uuid,
        tuple_placed_objects_dictionary,
        immutable_minor_composition_couple_type_boro_object_ckid: BoroObjectCkIds,
        owning_repository_uuid,
    ):
        bnop_tuple = BnopTuples(
            uuid,
            tuple_placed_objects_dictionary,
            immutable_minor_composition_couple_type_boro_object_ckid,
            owning_repository_uuid,
        )

        return bnop_tuple



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)
from bnop.migrations.bnop_to_xml_migration.xml_bnop_registers import (
    XmlBnopRegisters,
)


def sort_names_by_named_objects():
    sorted_bnop_names = list()

    for (
        bnop_object_uuid,
        bnop_object,
    ) in (
        BnopObjects.registry_keyed_on_uuid.items()
    ):
        for (
            bnop_name
        ) in bnop_object.is_named_bys:
            sorted_bnop_names.append(
                bnop_name
            )

    XmlBnopRegisters.update_migration_uow(
        boro_ckid_type=BoroObjectCkIds.Names,
        bnop_objects=sorted_bnop_names,
    )



from bnop.core.object_model.objects.bnop_names import (
    BnopNames,
)


class BnopNameFactories:

    @staticmethod
    def create(
        uuid,
        exemplar_representation,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        bnop_name = BnopNames(
            uuid,
            exemplar_representation,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        return bnop_name



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.migrations.bnop_to_xml_migration.adders.bnop_name_xml_adder import (
    add_bnop_name_to_xml_tree,
)
from bnop.migrations.bnop_to_xml_migration.common_knowledge.bnop_to_xml_migration_configurations import (
    BnopToXmlMigrationConfigurations,
)
from bnop.migrations.bnop_to_xml_migration.xml_bnop_registers import (
    XmlBnopRegisters,
)
from lxml import etree
from lxml.etree import Element


def add_names(
    xml_root: Element,
    configuration: BnopToXmlMigrationConfigurations,
):
    bnop_names = XmlBnopRegisters.get_migration_uow_type(
        boro_ckid_type=BoroObjectCkIds.Names
    )

    objects_tree_element = (
        etree.SubElement(
            xml_root, "Names"
        )
    )

    for bnop_name in bnop_names:
        add_bnop_name_to_xml_tree(
            bnop_name=bnop_name,
            parent_element=objects_tree_element,
            configuration=configuration,
        )



import shelve

from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)


def write_bnop(full_filename: str):
    bnop_shelf = shelve.open(
        full_filename
    )

    bnop_shelf.clear_xml_model_branches()

    for (
        key,
        value,
    ) in (
        BnopObjects.registry_keyed_on_uuid.items()
    ):
        bnop_shelf[key] = value

    bnop_shelf.close()


def read_bnop(full_filename: str):
    bnop_shelf = shelve.open(
        full_filename
    )

    for (
        key,
        value,
    ) in bnop_shelf.items():
        print(
            "Key: "
            + key
            + " - Type: "
            + str(type(value).__name__)
        )

    bnop_shelf.close()



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.core.object_model.objects.bnop_names import (
    BnopNames,
)
from bnop.migrations.bnop_to_xml_migration.adders.bnop_object_xml_adder import (
    add_bnop_object_types_to_xml_tree,
)
from bnop.migrations.bnop_to_xml_migration.common_knowledge.bnop_to_xml_migration_configurations import (
    BnopToXmlMigrationConfigurations,
)
from lxml import etree
from lxml.etree import Element


def add_bnop_name_to_xml_tree(
    bnop_name: BnopNames,
    parent_element: Element,
    configuration: BnopToXmlMigrationConfigurations,
):
    if (
        configuration.include_presentation_names
    ):
        name_tree_element = etree.SubElement(
            parent_element,
            BoroObjectCkIds.Names.name,
            uuid=str(bnop_name.uuid),
            uml_name=bnop_name.uml_name,
            name_exemplar=bnop_name.exemplar_representation,
        )

    else:
        name_tree_element = etree.SubElement(
            parent_element,
            BoroObjectCkIds.Names.name,
            uuid=str(bnop_name.uuid),
            name_exemplar=bnop_name.exemplar_representation,
        )

    __add_bnop_named_by_objects_to_xml_tree(
        bnop_name=bnop_name,
        name_tree_element=name_tree_element,
        configuration=configuration,
    )

    add_bnop_object_types_to_xml_tree(
        bnop_object=bnop_name,
        object_tree_element=name_tree_element,
        configuration=configuration,
    )


def __add_bnop_named_by_objects_to_xml_tree(
    bnop_name: BnopNames,
    name_tree_element: Element,
    configuration: BnopToXmlMigrationConfigurations,
):
    if len(bnop_name.names) == 0:
        return

    named_by_tree_element = etree.SubElement(
        name_tree_element,
        BoroObjectCkIds.NamedBy.name,
    )

    for (
        bnop_named_object
    ) in bnop_name.names:
        if (
            configuration.include_presentation_names
        ):
            etree.SubElement(
                named_by_tree_element,
                BoroObjectCkIds.Objects.name,
                uuid=str(
                    bnop_named_object.uuid
                ),
                uml_name=bnop_named_object.uml_name,
            )

        else:
            etree.SubElement(
                named_by_tree_element,
                BoroObjectCkIds.Objects.name,
                uuid=str(
                    bnop_named_object.uuid
                ),
            )



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bnop.core.object_model.objects.bnop_types import (
    BnopTypes,
)
from bnop.core.object_model.objects.places.bnop_placed_types_dictionaries import (
    BnopPlacedTypesDictionaries,
)


class BnopPlaceableTypes(BnopTypes):

    def __init__(
        self,
        uuid,
        placeable_type_placed_types_dictionary: BnopPlacedTypesDictionaries,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        BnopTypes.__init__(
            self,
            uuid,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        self.add_to_registry_keyed_on_ckid_type(
            BoroObjectCkIds.PlaceableTypes
        )

        self.placeable_type_placed_types_dictionary = placeable_type_placed_types_dictionary



from bnop.core.object_model.objects.bnop_type_places import (
    BnopTypePlaces,
)


class BnopTypePlaceFactories:

    @staticmethod
    def create(
        uuid,
        placing_type,
        placed_type,
        type_place_ckid,
        owning_repository_uuid,
    ):
        bnop_type_place = (
            BnopTypePlaces(
                uuid,
                placing_type,
                placed_type,
                type_place_ckid,
                owning_repository_uuid,
            )
        )

        return bnop_type_place



from bclearer_core.ckids.place_number_type_ckids import (
    PlaceNumberTypeCkIds,
)
from bnop.core.object_model.objects.bnop_types import (
    BnopTypes,
)


class BnopPlacedTypesDictionaries(
    object
):

    def __init__(self):
        self._placed_types_dictionary = (
            {}
        )

    def add_placed_type_to_dictionary(
        self,
        placing_ckid: PlaceNumberTypeCkIds,
        relation_placed_type: BnopTypes,
    ):
        self._placed_types_dictionary[
            placing_ckid
        ] = relation_placed_type

    def try_get_placed_type_for_place_number_ckid(
        self,
        placing_ckid: PlaceNumberTypeCkIds,
    ):
        placed_type = self._placed_types_dictionary[
            placing_ckid
        ]

        return placed_type



from bnop.core.object_model.objects.bnop_placeable_types import (
    BnopPlaceableTypes,
)


class BnopPlaceableTypeFactories:

    @staticmethod
    def create(
        uuid,
        placeable_type_placed_types_dictionary,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        bnop_placeabletype = BnopPlaceableTypes(
            uuid,
            placeable_type_placed_types_dictionary,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        return bnop_placeabletype



from bclearer_core.ckids.boro_object_ckids import (
    BoroObjectCkIds,
)
from bclearer_core.ckids.place_number_type_ckids import (
    PlaceNumberTypeCkIds,
)
from bclearer_orchestration_services.identification_services.uuid_service.uuid_helpers.uuid_factory import (
    create_new_uuid,
)
from bnop.core.factories.bnop_element_factories import (
    BnopElementFactories,
)
from bnop.core.factories.bnop_name_factories import (
    BnopNameFactories,
)
from bnop.core.factories.bnop_object_factories import (
    BnopObjectFactories,
)
from bnop.core.factories.bnop_placeabletype_factories import (
    BnopPlaceableTypeFactories,
)
from bnop.core.factories.bnop_tuple_factories import (
    BnopTupleFactories,
)
from bnop.core.factories.bnop_type_factories import (
    BnopTypeFactories,
)
from bnop.core.factories.bnop_type_place_factories import (
    BnopTypePlaceFactories,
)
from bnop.core.object_model.objects.bnop_names import (
    BnopNames,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)
from bnop.core.object_model.objects.places.bnop_placed_objects_dictionaries import (
    BnopPlacedObjectsDictionaries,
)
from bnop.core.object_model.objects.places.bnop_placed_types_dictionaries import (
    BnopPlacedTypesDictionaries,
)
from bnop.migrations.bnop_to_xml_migration.bnop_xml_write_orchestrator import (
    orchestrate_xml_write,
)


class BnopFacades:

    @staticmethod
    def write_bnop_object_to_xml(
        xml_file_path: str,
    ):
        orchestrate_xml_write(
            xml_file_path=xml_file_path
        )

    @staticmethod
    def create_new_bnop_type(
        owning_repository_uuid,
    ):
        bnop_type = (
            BnopTypeFactories.create(
                create_new_uuid(),
                owning_repository_uuid,
            )
        )

        return bnop_type

    @staticmethod
    def create_bnop_object(
        object_uuid,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        bnop_object = BnopObjectFactories.create(
            object_uuid,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        return bnop_object

    @staticmethod
    def create_bnop_type(
        type_uuid,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        bnop_type = BnopTypeFactories.create(
            type_uuid,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        return bnop_type

    @staticmethod
    def create_new_bnop_element(
        owning_repository_uuid,
    ):
        bnop_element = (
            BnopElementFactories.create(
                create_new_uuid(),
                owning_repository_uuid,
            )
        )

        return bnop_element

    @staticmethod
    def create_bnop_element(
        element_uuid,
        owning_repository_uuid,
    ):
        bnop_element = (
            BnopElementFactories.create(
                element_uuid,
                owning_repository_uuid,
            )
        )

        return bnop_element

    @staticmethod
    def create_new_bnop_name(
        exemplar_representation,
        owning_repository_uuid,
    ):
        bnop_name = (
            BnopNameFactories.create(
                create_new_uuid(),
                exemplar_representation,
                owning_repository_uuid,
            )
        )

        return bnop_name

    @staticmethod
    def create_bnop_name(
        name_uuid,
        exemplar_representation,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        bnop_name = BnopNameFactories.create(
            name_uuid,
            exemplar_representation,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        return bnop_name

    @staticmethod
    def create_new_bnop_tuple_from_two_placed_objects(
        placed1_object,
        placed2_object,
        immutable_minor_composition_couple_type_boro_object_ckid,
        owning_repository_uuid,
    ):
        tuple_placed_objects_dictionary = (
            BnopPlacedObjectsDictionaries()
        )

        tuple_placed_objects_dictionary.add_tuple_placed_object_to_dictionary(
            PlaceNumberTypeCkIds.PlaceNumberOnes,
            placed1_object,
        )

        tuple_placed_objects_dictionary.add_tuple_placed_object_to_dictionary(
            PlaceNumberTypeCkIds.PlaceNumberTwos,
            placed2_object,
        )

        bnop_tuple = BnopTupleFactories.create(
            create_new_uuid(),
            tuple_placed_objects_dictionary,
            immutable_minor_composition_couple_type_boro_object_ckid,
            owning_repository_uuid,
        )

        return bnop_tuple

    @staticmethod
    def create_bnop_tuple_from_two_placed_objects(
        tuple_uuid,
        placed1_object: BnopObjects,
        placed2_object: BnopObjects,
        immutable_minor_composition_couple_type_boro_object_ckid: BoroObjectCkIds,
        owning_repository_uuid,
    ):
        tuple_placed_objects_dictionary = (
            BnopPlacedObjectsDictionaries()
        )

        tuple_placed_objects_dictionary.add_tuple_placed_object_to_dictionary(
            PlaceNumberTypeCkIds.PlaceNumberOnes,
            placed1_object,
        )

        tuple_placed_objects_dictionary.add_tuple_placed_object_to_dictionary(
            PlaceNumberTypeCkIds.PlaceNumberTwos,
            placed2_object,
        )

        bnop_tuple = BnopTupleFactories.create(
            tuple_uuid,
            tuple_placed_objects_dictionary,
            immutable_minor_composition_couple_type_boro_object_ckid,
            owning_repository_uuid,
        )

        if (
            immutable_minor_composition_couple_type_boro_object_ckid
            == BoroObjectCkIds.SuperSubTypes
        ):
            placed2_object.supertypes.add(
                placed1_object
            )

            placed1_object.subtypes.add(
                placed2_object
            )

        if (
            immutable_minor_composition_couple_type_boro_object_ckid
            == BoroObjectCkIds.NamedBy
        ):
            placed1_object.is_named_bys.add(
                placed2_object
            )

            placed2_object.names.add(
                placed1_object
            )

        if (
            immutable_minor_composition_couple_type_boro_object_ckid
            == BoroObjectCkIds.TypesInstances
        ):
            placed2_object.types.add(
                placed1_object
            )

            placed1_object.instances.add(
                placed2_object
            )

            if (
                type(placed2_object)
                is BnopNames
            ):
                placed2_object.naming_spaces.add(
                    placed1_object
                )

        return bnop_tuple

    @staticmethod
    def create_bnop_placeabletype(
        placeabletype_uuid,
        owning_repository_uuid,
        presentation_name=str(),
    ):
        placeable_type_placed_type_dictionary = (
            BnopPlacedTypesDictionaries()
        )

        bnop_placeabletype = BnopPlaceableTypeFactories.create(
            placeabletype_uuid,
            placeable_type_placed_type_dictionary,
            owning_repository_uuid,
            presentation_name=presentation_name,
        )

        return bnop_placeabletype

    @staticmethod
    def create_bnop_type_place(
        type_place_uuid,
        placing_type,
        placed_type,
        type_place_ckid,
        owning_repository_uuid,
    ):
        bnop_type_place = BnopTypePlaceFactories.create(
            type_place_uuid,
            placing_type,
            placed_type,
            type_place_ckid,
            owning_repository_uuid,
        )

        return bnop_type_place

    @staticmethod
    def __add_placed_type_to_placeabletype(
        placed_type,
        place_number_ckid,
        placeable_type,
    ):
        placeable_type.placeable_type_placed_types_dictionary[
            place_number_ckid
        ] = placed_type



from bclearer_core.ckids.place_number_type_ckids import (
    PlaceNumberTypeCkIds,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)


class BnopPlacedObjectsDictionaries(
    object
):

    def __init__(self):
        self.relation_placed_object_dictionary = (
            {}
        )

    def add_tuple_placed_object_to_dictionary(
        self,
        placing_ckid: PlaceNumberTypeCkIds,
        relation_placed_object: BnopObjects,
    ):
        self.relation_placed_object_dictionary[
            placing_ckid
        ] = relation_placed_object

    def try_get_relation_placed_object_for_place_number_ckid(
        self,
        placing_ckid: PlaceNumberTypeCkIds,
    ):
        relation_placed_object = self.relation_placed_object_dictionary[
            placing_ckid
        ]

        return relation_placed_object



from bnop.migrations.bnop_to_xml_migration.adders.bnop_xml_named_objects_adder import (
    add_named_objects,
)
from bnop.migrations.bnop_to_xml_migration.adders.bnop_xml_names_adder import (
    add_names,
)
from bnop.migrations.bnop_to_xml_migration.common_knowledge.bnop_to_xml_migration_configurations import (
    BnopToXmlMigrationConfigurations,
)
from bnop.migrations.bnop_to_xml_migration.sorters.bnop_name_sorters import (
    sort_names_by_named_objects,
)


class VanillaBnopToXmlMigrationConfigurations(
    BnopToXmlMigrationConfigurations
):
    include_presentation_names = False

    xml_section_creators = [
        add_named_objects,
        add_names,
    ]

    bnop_object_sorters = [
        sort_names_by_named_objects
    ]



from bclearer_core.ckids.place_number_type_ckids import (
    PlaceNumberTypeCkIds,
)
from bnop.core.object_model.objects.bnop_objects import (
    BnopObjects,
)
from bnop.core.object_model.objects.bnop_types import (
    BnopTypes,
)


class BnopTypePlaces(BnopObjects):

    def __init__(
        self,
        uuid,
        placing_type: BnopTypes,
        placed_type: BnopTypes,
        type_place_ckid: PlaceNumberTypeCkIds,
        owning_repository_uuid,
    ):
        BnopObjects.__init__(
            self,
            uuid,
            owning_repository_uuid,
        )

        self.placing_type = placing_type

        self.placed_type = placed_type

        self.type_place_ckid = (
            type_place_ckid
        )
